<?php

/**
 * Implements hook_drush_help().
 */
function locale_drush_help($section) {
  switch ($section) {
    case 'drush:locale-dump':
      return dt('Exports the Drupal Locale as PO files.');
  }
}

/**
 * Implementation of hook_drush_command().
 */
function locale_drush_command() {
  $items = array();
  $items['locale-dump'] = array(
    'callback' => 'drush_locale_dump',
    'description' => 'Exports the Drupal interface translations as Gettext Portable Object (.po) files.',
    'examples' => array(
      'drush locale-dump' => 'Save all exported files in a date based directory that will be created under ~/drush-backups directory.',
    ),
    'options' => array(
      'result-directory' => array(
        'description' => 'Save to a directory. The directory should be writable by drush. If --result-directory is omitted then date based directory will be created under ~/drush-backups directory.',
        'example-value' => '/path/to/file',
        'value' => 'optional',
      ),
    ),
  );

  $items['locale-import'] = array(
    'callback' => 'drush_locale_import',
    'description' => 'Imports a Gettext Portable Object (.po) file.',
    'examples' => array(
      'drush locale-import project.po nl default' => 'Import a Gettext Portable Object (.po) file into the dutch default group.',
    ),
    'arguments' => array(
      'file' => array(
        'description' => 'Gettext Portable Object (.po) file to import.',
        'example-value' => '/path/to/file.po',
      ),
      'language' => array(
        'description' => 'The language of the Gettext Portable Object (.po) file to import.',
        'example-value' => 'fr',
      ),
      'group' => array(
        'description' => 'The text group of the Gettext Portable Object (.po) file to import.',
        'example-value' => 'default',
      ),
    ),
    'options' => array(
      'import-mode' => array(
        'description' => 'Set the mode for importing. Setting this to 0 will overwrite all existing strings, 1 will keep existing strings. Omitting this value means that the default value of the importing site will be used. This is usually overwrite but with variable locale_sync_default_mode the default can be overridden.',
        'example-value' => '1',
        'value' => 'optional',
      ),
    ),
  );

  $items['locale-sync'] = array(
    'callback' => 'drush_locale_sync',
    'bootstrap' => DRUSH_BOOTSTRAP_DRUSH,
    'description' => 'Copy and import interface translations from one site into another site. Transfers via rsync.',
    'drush dependencies' => array('core'), // core-rsync.
    'examples' => array(
      'drush locale-sync @dev @prod' => 'Copy the translations defined in @dev to @prod.',
    ),
    'arguments' => array(
      'from' => 'Name of subdirectory within /sites or a site-alias.',
      'to' => 'Name of subdirectory within /sites or a site-alias.',
    ),
    'options' => array(
      'import-mode' => array(
        'description' => 'Set the mode for importing. Setting this to 0 will overwrite all existing strings, 1 will keep existing strings. Omitting this value means that the default value of the importing site will be used. This is usually overwrite but with variable locale_sync_default_mode the default can be overridden.',
        'example-value' => '1',
        'value' => 'optional',
      ),
    ),
  );

  return $items;
}

/**
 * Command callback. Outputs all Drupal textgroups in all different languages.
 */
function drush_locale_dump() {
  $result_directory = drush_get_option('result-directory', FALSE);

  if ($result_directory && !file_exists($result_directory)) {
    mkdir($result_directory);
  }

  $languages = language_list();

  $names = locale_language_list('name');
  unset($names['en']);

  // Determine result directory, if not set.
  if (empty($result_directory)) {
    // User did not pass a specific value for --result-file. Make one.
    $backup = drush_include_engine('version_control', 'backup');
    $result_directory = $backup->prepare_backup_dir('locale');
    if (empty($result_directory)) {
      $result_directory = drush_find_tmp();
    }
  }

  $groups = module_invoke_all('locale', 'groups');
  $date = gmdate('Ymd_his');

  $files = array();

  // Dump each language + group in a file.
  foreach ($names as $langcode => $language_name) {
    foreach ($groups as $group => $group_name) {
      // $result_directory is passed in to us usually via --result-directory.  If the user
      // has set $options['result-directory'] = TRUE, then we
      // will generate PO dump files in the same backup
      // directory that pm-updatecode uses.

      $strings = _locale_export_get_strings($languages[$langcode], $group);
      $output = _locale_export_po_generate($languages[$langcode], $strings);

      $filename = drush_get_context('DRUSH_DRUPAL_SITE') . '-' . $langcode . '-' . $date . '-' . $group . '.po';
      $filepath = $result_directory . '/' . $filename;
      file_put_contents($filepath, $output);

      $files[$langcode][$group] = array('filename' => $filename, 'filepath' => $filepath);

      drush_log(dt("Wrote .po file @file for @language - @group", array('@file' => $filepath, '@language' => $language_name, '@group' => $group_name)), 'ok');
    }
  }
  return $files;
}

/**
 * Command callback. Imports a Gettext Portable Object (.po) file into a Drupal text group for a set language.
 */
function drush_locale_import($filename, $langcode, $group) {
  // Skip taxonomy group because these translations have vocabulary id's and
  // term id's and these are probably not the same on different environments.
  if (in_array($group, variable_get('locale_sync_skip_groups', array('menu', 'taxonomy')))) {
    return;
  }

  $default = variable_get('locale_sync_default_mode', LOCALE_IMPORT_OVERWRITE);
  $mode = drush_get_option('import-mode', $default);
  $mode = $mode == -1 ? $default : $mode;

  $languages = language_list('language');

  if (!isset($languages[$langcode])) {
    drush_log(dt("Language @language does not exist in your Drupal setup.", array('@language' => $langcode)), 'error');
    return;
  }

  // Store the dump into managed files table.
  $input = file_get_contents($filename);
  $input && $file = file_save_data($input);

  // Import file into Drupal.
  drush_log(dt("Importing po file @file into @language - @group.", array('@file' => $filename, '@language' => $langcode, '@group' => $group)), 'status');

  if ($file && _locale_import_po($file, $langcode, $mode, $group)) {
    // Output logs, otherwise they will all be shown on the end of the drush call.
    _drush_log_drupal_messages();
  }
  else {
    drush_log(dt("Po file @file failed to import into your Drupal installation.", array('@file' => $filename)), 'error');
  }
  $file && file_delete($file);
}

/**
 * Command callback. Sync Drupal locales.
 */
function drush_locale_sync($source = NULL, $target = NULL) {
  $mode = drush_get_option('import-mode', -1); // -1 == means let the importing site decide what the default is.

  // Preflight destination in case it defines the alias used by the source
  _drush_sitealias_get_record($target);

  // After preflight, get source and destination settings
  $source_settings = drush_sitealias_get_record($source);
  $source_db_url = drush_sitealias_get_db_spec($source_settings, FALSE, 'source-');
  $target_settings = drush_sitealias_get_record($target);
  $target_db_url = drush_sitealias_get_db_spec($target_settings, FALSE, 'source-');

  $txt_source = (isset($source_db_url['remote-host']) ? $source_db_url['remote-host'] . '/' : '') . $source_db_url['database'];
  $txt_target = (isset($target_db_url['remote-host']) ? $target_db_url['remote-host'] . '/' : '') . $target_db_url['database'];

  drush_print();
  if (drush_get_context('DRUSH_NOCOLOR')) {
    drush_print(dt("You will copy data from @source to @target.", array('@target' => $txt_target, '@source' => $txt_source)));
  }
  else {
    drush_print(dt("You will copy data from \033[1;32;40m\033[1m@source\033[0m to \033[1;32;40m\033[1m@target\033[0m.", array('@target' => $txt_target, '@source' => $txt_source)));
  }
  drush_print();
  if (!drush_confirm(dt('Do you really want to continue?'))) {
    return drush_user_abort();
  }
  require_once DRUSH_BASE_PATH . '/commands/core/rsync.core.inc';

  $additional_options = array();

  // Get remote tmp directories. Extends *_settings, doesn't override them.
  $source_settings = drush_sitealias_evaluate_path($source . ':%temp', $additional_options, FALSE, "rsync", 'source-');
  $target_settings = drush_sitealias_evaluate_path($target . ':%temp', $additional_options, FALSE, "rsync", 'target-');

  $date = gmdate('Ymd_his');

  // Set remote dump directory.
  $source_dump_directory = $source_settings['path-aliases']['%temp'] . '/' . $date;

  // Set rsync remote dump directories.
  $source_path = $source_settings['evaluated-path'] . '/' . $date;
  $target_path = $target_settings['evaluated-path'];

  // Create dump.
  $result = drush_invoke_process($source, 'locale-dump', $additional_options, array('result-directory' => $source_dump_directory), array('integrate' => FALSE));

  // Rsync dump.
  $options = _drush_build_rsync_options($additional_options);

  // Detect if both source and target are remote.
  $source_db_url = drush_sitealias_get_db_spec($source_settings, FALSE, 'source-');
  $target_db_url = drush_sitealias_get_db_spec($target_settings, FALSE, 'target-');
  if (isset($source_db_url['remote-host']) && isset($target_db_url['remote-host'])) {
    $temp_dir = drush_find_tmp() . '/';
    drush_core_exec_rsync($source_path, $temp_dir, $options);
    drush_core_exec_rsync($temp_dir . $date, $target_path, $options);
  }
  else {
    drush_core_exec_rsync($source_path, $target_path, $options);
  }

  // Import dump.
  $error = FALSE;
  foreach ($result['object'] as $langcode => $groups) {
    foreach ($groups as $group_name => $file) {
      $result = drush_invoke_process($target, 'locale-import', array('file' => $target_settings['path'] . '/' . $date . '/' . $file['filename'], 'language' => $langcode, 'group' => $group_name), array('import-mode' => $mode));

      $error |= $result['error_status'] == 1;
    }
  }

  if ($error) {
    drush_log(dt("Syncing the sites locale failed"), 'error');
  }
  else {
    drush_log(dt("Finished syncing the sites locale"), 'ok');
  }
}

